@@ -19,7 +19,10 @@
 /* Public API for other Nginx modules */
 
 
-#define ngx_http_lua_version  10023
+#define ngx_http_lua_version  10026
+
+
+typedef struct ngx_http_lua_co_ctx_s  ngx_http_lua_co_ctx_t;
 
 
 typedef struct {
@@ -56,6 +59,17 @@ ngx_shm_zone_t *ngx_http_lua_find_zone(u_char *name_data, size_t name_len);
 ngx_shm_zone_t *ngx_http_lua_shared_memory_add(ngx_conf_t *cf, ngx_str_t *name,
     size_t size, void *tag);
 
+ngx_http_lua_co_ctx_t *ngx_http_lua_get_cur_co_ctx(ngx_http_request_t *r);
+
+void ngx_http_lua_set_cur_co_ctx(ngx_http_request_t *r,
+    ngx_http_lua_co_ctx_t *coctx);
+
+lua_State *ngx_http_lua_get_co_ctx_vm(ngx_http_lua_co_ctx_t *coctx);
+
+void ngx_http_lua_co_ctx_resume_helper(ngx_http_lua_co_ctx_t *coctx, int nrets);
+
+int ngx_http_lua_get_lua_http10_buffering(ngx_http_request_t *r);
+
 
 #endif /* _NGX_HTTP_LUA_API_H_INCLUDED_ */
 
