@@ -55,11 +55,17 @@ typedef struct {
 
 
 #if (NGX_PCRE)
-#include <pcre.h>
-#   if (PCRE_MAJOR > 8) || (PCRE_MAJOR == 8 && PCRE_MINOR >= 21)
+#   if (NGX_PCRE2)
 #       define LUA_HAVE_PCRE_JIT 1
 #   else
-#       define LUA_HAVE_PCRE_JIT 0
+
+#include <pcre.h>
+
+#       if (PCRE_MAJOR > 8) || (PCRE_MAJOR == 8 && PCRE_MINOR >= 21)
+#           define LUA_HAVE_PCRE_JIT 1
+#       else
+#           define LUA_HAVE_PCRE_JIT 0
+#       endif
 #   endif
 #endif
 
@@ -221,9 +227,14 @@ struct ngx_http_lua_main_conf_s {
     ngx_int_t            regex_cache_entries;
     ngx_int_t            regex_cache_max_entries;
     ngx_int_t            regex_match_limit;
-#   if (LUA_HAVE_PCRE_JIT)
+#endif
+
+#if (LUA_HAVE_PCRE_JIT)
+#if (NGX_PCRE2)
+    pcre2_jit_stack     *jit_stack;
+#else
     pcre_jit_stack      *jit_stack;
-#   endif
+#endif
 #endif
 
     ngx_array_t         *shm_zones;  /* of ngx_shm_zone_t* */
@@ -360,6 +371,8 @@ union ngx_http_lua_srv_conf_u {
 typedef struct {
 #if (NGX_HTTP_SSL)
     ngx_ssl_t              *ssl;  /* shared by SSL cosockets */
+    ngx_array_t            *ssl_certificates;
+    ngx_array_t            *ssl_certificate_keys;
     ngx_uint_t              ssl_protocols;
     ngx_str_t               ssl_ciphers;
     ngx_uint_t              ssl_verify_depth;
@@ -497,6 +510,7 @@ struct ngx_http_lua_co_ctx_s {
     unsigned                 nresults_from_worker_thread;  /* number of results
                                                             * from worker
                                                             * thread callback */
+    unsigned                 nrets;     /* ngx_http_lua_run_thread nrets arg. */
 
     unsigned                 nsubreqs;  /* number of subrequests of the
                                          * current request */
@@ -586,7 +600,7 @@ typedef struct ngx_http_lua_ctx_s {
     ngx_chain_t             *filter_in_bufs;  /* for the body filter */
     ngx_chain_t             *filter_busy_bufs;  /* for the body filter */
 
-    ngx_http_cleanup_pt     *cleanup;
+    ngx_pool_cleanup_pt     *cleanup;
 
     ngx_http_cleanup_t      *free_cleanup; /* free list of cleanup records */
 
